precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D maskTexture;
uniform sampler2D dstTexture;

uniform int needNormalize;
uniform int premultiplied;

uniform vec2 center;
uniform float effectValue;

lowp vec4 unpremultiply(lowp vec4 s) {
    return premultiplied > 0 ? vec4(s.rgb/max(s.a, 0.00001), s.a) : s;
}

lowp vec4 premultiply(lowp vec4 s) {
    return premultiplied > 0 ? vec4(s.rgb * s.a, s.a) : s;
}

lowp vec4 normalBlend(lowp vec4 Cb, lowp vec4 Cs) {
    lowp vec4 dst = premultiply(Cb);
    lowp vec4 src = premultiply(Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}

void main()
{
    vec2 uv = textureCoordinate;
    vec2 srcUv = uv;
    srcUv -= center;
    srcUv /= 1.0 + effectValue;
    srcUv += center;
    vec4 srcCol = texture2D(inputImageTexture, srcUv);
    vec4 dstCol = texture2D(dstTexture, uv);
    float weight = texture2D(maskTexture, uv).r;

    vec4 outCol = vec4(0.0);
    outCol = premultiply(srcCol) * weight + premultiply(dstCol);
    outCol = unpremultiply(outCol);

    if (needNormalize == 1) {
        float w = max(outCol.a, 1.0);
        outCol /= w;
        outCol = normalBlend(vec4(1.0), outCol);
    }
//    vec4 outCol = vec4(0.0);
//    if(weight < 0.9 && weight > 0.2) {
//        outCol = vec4(1.0);
//    }
    gl_FragColor = outCol;
}