precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform vec2 screenSize;
uniform vec2 imageSize;
uniform vec2 startPos;
uniform vec4 bodyRect;
uniform int splicingMode;//0,无拼接；1,上下拼接；2,左右拼接
uniform int colorChangeMode;//0,默认；1，color blend;2, hsv blend
uniform vec3 normalColor;
uniform vec3 datumColor;
uniform int  rotationAngle;
uniform int flipX;
uniform int flipY;
//only support 0,90,180,270 clockwise rotation，flip before rotation
vec4 unpremultiply(vec4 s) {
    return vec4(s.rgb/max(s.a,0.00001), s.a);
}

vec4 premultiply(vec4 s) {
    return vec4(s.rgb * s.a, s.a);
}

vec4 normalBlend(vec4 Cb, vec4 Cs) {
    vec4 dst = premultiply(Cb);
    vec4 src = premultiply(Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}

vec3 saturate(vec3 rgb) {
    return clamp(rgb, vec3(0.0), vec3(1.0));
}

vec4 saturate2(lowp vec4 v) {
    return clamp(v, 0.0, 1.0);
}

vec4 blendBaseAlpha(vec4 Cb, vec4 Cs, vec4 B) {
    vec4 Cr = vec4((1.0 - Cb.a) * Cs.rgb + Cb.a * saturate(B.rgb), Cs.a);
    return normalBlend(Cb, Cr);
}

// multiply
vec4 multiplyBlend(vec4 Cb, vec4 Cs) {
    vec4 B = saturate2(vec4(Cb.rgb * Cs.rgb, Cs.a));
    return blendBaseAlpha(Cb, Cs, B);
}

vec4 screenBlend(vec4 Cb, vec4 Cs) {
    vec4 White = vec4(1.0);
    vec4 B = White - ((White - Cs) * (White - Cb));
    return blendBaseAlpha(Cb, Cs, B);
}

vec4 addBlend(vec4 Cb, vec4 Cs) {
    vec4 B = min(Cb + Cs, 1.0);
    return blendBaseAlpha(Cb, Cs, B);
}

vec4 getIconColor(sampler2D texture, vec2 uv, int mode) {
    if(mode == 0) {
        return texture2D(texture, uv);
    } else if(mode == 1) {
        vec4 col0 = texture2D(texture, vec2(uv.x, uv.y/2.0));
        vec4 col1 = texture2D(texture, vec2(uv.x, 0.5 + uv.y/2.0));
        return vec4(col1.rgb, col0.g);
    } else if(mode == 2) {
        vec4 col0 = texture2D(texture, vec2(uv.x / 2.0, uv.y));
        vec4 col1 = texture2D(texture, vec2(0.5 + uv.x / 2.0, uv.y));
        return vec4(col1.rgb, col0.g);
    } else {
        return vec4(0.0);
    }
}

vec3 rgb2hsv(vec3 color) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(color.bg, K.wz), vec4(color.gb, K.xy), step(color.b, color.g));
    vec4 q = mix(vec4(p.xyw, color.r), vec4(color.r, p.yzx), step(p.x, color.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 color) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(color.xxx + K.xyz) * 6.0 - K.www);
    return color.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), color.y);
}

float lum(vec4 color) {
    return dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
}

vec4 clipColor(vec4 C) {
    float l = lum(C);
    float n = min(C.r, min(C.g, C.b));
    float x = max(C.r, max(C.g, C.b));
    if (n < 0.0) {
        return vec4((l + ((C.rgb - l) * l) / (l - n)), C.a);
    }
    if (x > 1.0) {
        return vec4(l + (((C.rgb - l) * (1.0 - l)) / (x - l)), C.a);
    }
    return C;
}

vec4 setLum(vec4 C, float l) {
    float d = l - lum(C);
    return clipColor(vec4(C.rgb + d, C.a ));
}

vec4 colorBlend(vec4 Cb, vec4 Cs) {
    vec4 B = setLum(Cs, lum(Cb));
    return blendBaseAlpha(Cb, Cs, B);
}

vec4 chengeEffectColor(int mode, vec3 normalColor, vec3 datumColor, vec4 inputColor) {
    vec4 outCol = inputColor;
    if(mode == 1) {
        outCol = colorBlend(inputColor, vec4(normalColor, 1.0));
        outCol.a = lum(outCol);
        outCol.rgb /= outCol.a;
    } else if(mode == 2) {
        vec3 datumHueColor = rgb2hsv(datumColor);
        vec3 hueColor = rgb2hsv(normalColor);
        float changeColor = datumHueColor.r - hueColor.r;

        vec3 inputHsvCol = rgb2hsv(inputColor.rgb);
        vec3 outputHsvCol = vec3(inputHsvCol.x - changeColor, inputHsvCol.y, inputHsvCol.z);
        outCol = vec4(hsv2rgb(outputHsvCol), inputColor.a);

        vec4 AlphaRundingColor = vec4(hsv2rgb(vec3(inputHsvCol.x ,0.0,inputHsvCol.z)), inputColor.a);
        outCol.a = AlphaRundingColor.r;
        outCol.rgb /= outCol.a;
    }
    //outCol.a = max(outCol.r, max(outCol.g, outCol.b));
    return outCol;
}
void main()
{
    vec2 uv  = textureCoordinate;
    uv.y = 1.0 - uv.y;//left,top is (0,0)

    vec2 imageUv  = uv * screenSize;

    vec4 outCol = vec4(0.0);

    if(imageUv.x >= startPos.x && imageUv.x <= startPos.x + imageSize.x
    && imageUv.y >= startPos.y && imageUv.y <= startPos.y + imageSize.y) {
        vec2 effectImgUv;
        effectImgUv.x = (imageUv.x - startPos.x) / imageSize.x;
        effectImgUv.y = (imageUv.y - startPos.y) / imageSize.y;
        if(rotationAngle == 90) {
            vec2 newImgUv = vec2(0.0);
            newImgUv.x = 1.0 - effectImgUv.y;
            newImgUv.y = effectImgUv.x;
            effectImgUv = newImgUv;
        } else if (rotationAngle == 180) {
            effectImgUv.x = 1.0 - effectImgUv.x;
            effectImgUv.y = 1.0 - effectImgUv.y;
        } else if (rotationAngle == 270) {
            vec2 newImgUv = vec2(0.0);
            newImgUv.x = effectImgUv.y;
            newImgUv.y = 1.0 - effectImgUv.x;
            effectImgUv = newImgUv;
        }

        if(flipX > 0) effectImgUv.x = 1.0 - effectImgUv.x;
        if(flipY > 0) effectImgUv.y = 1.0 - effectImgUv.y;

        vec4 effectColor  = getIconColor(inputImageTexture, effectImgUv, splicingMode);
        outCol = chengeEffectColor(colorChangeMode, normalColor, datumColor, effectColor);
    }

    gl_FragColor = outCol;
}