precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D maskTexture;
uniform sampler2D backgroundTexture;

uniform int needNormalize;
uniform int premultiplied;

lowp vec4 unpremultiply(lowp vec4 s) {
    return premultiplied > 0 ? vec4(s.rgb/max(s.a, 0.00001), s.a) : s;
}

lowp vec4 premultiply(lowp vec4 s) {
    return premultiplied > 0 ? vec4(s.rgb * s.a, s.a) : s;
}

lowp vec4 normalBlend(lowp vec4 Cb, lowp vec4 Cs) {
    lowp vec4 dst = premultiply(Cb);
    lowp vec4 src = premultiply(Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}

vec3 saturate(vec3 v) {
    return clamp(v, 0.0, 1.0);
}

vec4 blendBaseAlpha(vec4 Cb, vec4 Cs, vec4 B) {
    vec4 Cr = vec4((1.0 - Cb.a) * Cs.rgb + Cb.a * saturate(B.rgb), Cs.a);
    return normalBlend(Cb, Cr);
}

vec4 screenBlend(vec4 Cb, vec4 Cs) {
    vec4 White = vec4(1.0);
    vec4 B = White - ((White - Cs) * (White - Cb));
    return blendBaseAlpha(Cb, Cs, B);
}

void main()
{
    vec2 uv = textureCoordinate;
    vec4 srcCol = texture2D(inputImageTexture, uv);
    vec4 backCol = texture2D(backgroundTexture, uv);
    vec4 maskCol = texture2D(maskTexture, uv);

    vec4 resultColor =  backCol * maskCol.r * 1.2;
    resultColor = screenBlend(resultColor, srcCol);

    gl_FragColor = resultColor;
}